/////////////////////////////////////////////////////////////////////////////////////

// Requirement

const path = require('path')
const MiniCssExtractPlugin = require('mini-css-extract-plugin')
const FixStyleOnlyEntriesPlugin = require('webpack-fix-style-only-entries')
const OptimizeCssAssetsPlugin = require('optimize-css-assets-webpack-plugin')
const TerserPlugin = require('terser-webpack-plugin')
const HtmlWebpackPlugin = require('html-webpack-plugin')

// postcss plugins
const postCssCombineDuplicatedSelectors = require('postcss-combine-duplicated-selectors')
const cssDeclarationSorter = require('css-declaration-sorter')
// const cssMqpacker = require('@lipemat/css-mqpacker')
const cssMqpacker = require('css-mqpacker')
const sortCssMediaQueries = require('sort-css-media-queries')
const autoprefixer = require('autoprefixer')

/////////////////////////////////////////////////////////////////////////////////////

// Config

const config = require('./config')
const dir = config.dir
const devMode = process.env.NODE_ENV !== 'production'
/////////////////////////////////////////////////////////////////////////////////////

// Exports

module.exports = {
  entry: {
    motiv: path.resolve(__dirname, dir.src, 'js', 'motiv.js'),
    demo: path.resolve(__dirname, dir.src, 'js', 'demo.js'),
    'css/motiv.css': path.resolve(__dirname, dir.src, 'stylus', 'style.styl'),
    'css/demo.css': path.resolve(__dirname, dir.src, 'stylus', 'demo.styl')
  },

  output: {
    filename: 'js/[name].js',
    path: path.resolve(__dirname, dir.dest)
  },

  resolve: {
    modules: [
      'node_modules',
      'modules'
    ],
    alias: {
      wordpress: path.resolve(__dirname, '../../plugins')
    }
  },

  cache: devMode,

  externals: {
    jquery: 'jQuery'
  },

  optimization: {
    minimize: !devMode,
    minimizer: [
      new OptimizeCssAssetsPlugin(),
      new TerserPlugin({
        cache: true,
        parallel: true,
        terserOptions: {
          ecma: 6,
          compress: true,
          output: {
            beautify: false,
            comments: false
          }
        }
      })
    ]
  },

  plugins: [
    new FixStyleOnlyEntriesPlugin({
      extensions: ['styl', 'css']
    }),
    new MiniCssExtractPlugin({
      filename: '[name]'
    }),
    new HtmlWebpackPlugin({
      title: 'FrontPage',
      template: path.resolve(__dirname, dir.src, 'pug', 'index.pug'),
      excludeChunks: ['css/motiv.css']
    }),
    new HtmlWebpackPlugin({
      title: 'Foundation',
      template: path.resolve(__dirname, dir.src, 'pug', 'foundation.pug'),
      filename: 'foundation.html',
      excludeChunks: ['css/motiv.css']
    }),
    new HtmlWebpackPlugin({
      title: 'GetStarted',
      template: path.resolve(__dirname, dir.src, 'pug', 'get-started.pug'),
      filename: 'get-started.html',
      excludeChunks: ['css/motiv.css']
    }),
    new HtmlWebpackPlugin({
      title: 'Layouts',
      template: path.resolve(__dirname, dir.src, 'pug', 'layouts.pug'),
      filename: 'layouts.html',
      excludeChunks: ['css/motiv.css']
    }),
    new HtmlWebpackPlugin({
      title: 'Utility Classes',
      template: path.resolve(__dirname, dir.src, 'pug', 'utilities.pug'),
      filename: 'utilities.html',
      excludeChunks: ['css/motiv.css']
    }),
    new HtmlWebpackPlugin({
      title: 'Mixins',
      template: path.resolve(__dirname, dir.src, 'pug', 'mixins.pug'),
      filename: 'mixins.html',
      excludeChunks: ['css/motiv.css']
    }),
    new HtmlWebpackPlugin({
      title: 'Components',
      template: path.resolve(__dirname, dir.src, 'pug', 'components.pug'),
      filename: 'components.html',
      excludeChunks: ['css/motiv.css']
    })
  ],

  module: {
    rules: [
      {
        test: /\.js$/,
        use: [
          {
            loader: 'babel-loader',
            options: {
              presets: ['@babel/preset-env']
            }
          }
        ]
      },
      // {
      //   test: /\.pug$/,
      //   use: ExtractTextPlugin.extract([
      //     {
      //       loader: 'html-loader'
      //     },
      //     {
      //       loader: 'pug-plain-loader',
      //       options: options.pugLoader
      //     }
      //   ])
      // },
      {
        test: /\.pug$/,
        use: [
          {
            loader: 'pug-loader',
            options: {
              pretty: devMode,
              root: path.resolve(__dirname, dir.src, 'pug')
            }
          }
        ]
      },
      {
        test: /\.(styl|css)$/,
        use: [
          {
            loader: MiniCssExtractPlugin.loader
          },
          {
            loader: 'css-loader'
          },
          {
            loader: 'postcss-loader',
            options: {
              ident: 'postcss',
              plugins: () => [
                postCssCombineDuplicatedSelectors({
                  removeDuplicatedProperties: true
                }),
                cssMqpacker({
                  // sort: sortCssMediaQueries
                  sort: true
                }),
                cssDeclarationSorter({
                  order: 'smacss'
                }),
                autoprefixer({
                  grid: true
                })
              ]
            }
          },
          {
            loader: 'resolve-url-loader'
          },
          {
            loader: 'stylus-loader',
            options: {
              'include css': true
            }
          }
        ]
      },
      {
        // test: /\.(jpe?g|png|gif|svg)$/,
        test: /\.(jpe?g|png|gif|svg|ico)(\?.+)?$/,
        use: [
          {
            loader: 'url-loader',
            options: {
              limit: 8192,
              name: path.join('images', '[name].[ext]')
            }
          },
          {
            loader: 'img-loader' // options あとでやる
          }
        ]
      },
      {
        // test: /\.(ttf|otf|eot|woff(2)?)(\?[a-z0-9]+)?$/,
        test: /\.(eot|otf|ttf|woff2?|svg)(\?.+)?$/,
        use: [
          {
            loader: 'file-loader',
            options: {
              name: path.join('fonts', '[name].[ext]')
            }
          }
        ]
      }
    ]
  }
}
